// Part of CAPer, Main Part, for NS 3.3pl1
// Frank Siegert 1996, may be used freely for any purpose 
// except commercial distribution for which you need my written consent

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>  
#include <regex.h>

#include <pwd.h>

#import <nikit/NIDomain.h>
#import <nikit/NILoginPanel.h>
#import "Subprocess.h"
#import "ShowTextControler.h"
#import "CAPInstallObject.h"

#define STARTUP_SCRIPT "/usr/local/cap/start-cap-servers"
#define PRINTER_DIR "/usr/local/lib/cap"
#define AFP_FILE "/usr/local/lib/cap/afpvols"
#define ZONEFILE "/usr/local/lib/cap/active_zone"
#define TMP_DIR "/tmp"

#define FILESERVERLOG "/usr/local/lib/cap/AUFS.log"
#define PRINTSERVERLOG "/usr/local/lib/cap/LWSRV.log"

@implementation CAPStorage

- init
{
	[super init];
	name=NULL;
	data=NULL;
	reference=NULL;
	return self;
}

- free
{
	if (name) free(name);
	if (data) free(data);
	return [super free];
}

- setName:(char *)newname
{
	if (name) free(name);
	name=NULL;
	if (!newname) return self;
	name=(char *)malloc(strlen(newname)+2);
	if (!name) return NULL;
	strcpy(name, newname);
	return self;
}

- setStatus:(int)newstatus
{
	status=newstatus;
	return self;
}

- (int)status
{
	return status;
}

- setData:(char *)newdata
{
	if (data) free(data);
	data=NULL;
	if (!newdata) return self;
	data=(char *)malloc(strlen(newdata)+2);  // More to be safe
	if (!data) return NULL;
	strcpy(data,newdata);
	return self;
}

- setRef:(id)myid
{
	reference=myid;
	return self;
}

- ref
{
	return reference;
}

- (char *)data
{
	return data;
}

- (char *)name
{
	return name;
}

@end

@implementation CAPInstallObject


// special for opening a script, takes tag from sender and does open depending on this value

- openScript:sender
{
    char buffer[1024];
    DIR *pdir;
    struct direct *dp;
    id appSpeaker = [[Speaker alloc] init];
    int appPort;
    
    appPort=NXPortFromName("Edit",NULL);
    buffer[0]=0;
    switch ([[sender selectedCell] tag]) {
	case 1:
    		sprintf(buffer,"%s",STARTUP_SCRIPT);
		break;
	case 2:
  		pdir=opendir(PRINTER_DIR);
		if (!pdir) return self;
          	for (dp = readdir(pdir); dp != NULL; dp = readdir(pdir)) 
             	{
			if (!recmp(".conf",dp->d_name)) {
				sprintf(buffer,"%s/%s", PRINTER_DIR,dp->d_name);
    				[appSpeaker setSendPort:appPort];
    				[appSpeaker selectorRPC:"openFile:onHost:atTrueLine:"
	     			paramTypes:"cci", buffer, "localhost", 0];
			}
		}
	        closedir(pdir);
    		[appSpeaker performRemoteMethod:"unhide"];
    		[appSpeaker free];
    		return self;
		break;
	case 3:
    		sprintf(buffer,"%s",AFP_FILE);
		break;
    }
    if (!appPort) {
	NXRunAlertPanel(NULL, "Unable to connect to Edit", "Ok",NULL,NULL);
    	[appSpeaker free];
	return self;
    }
    [appSpeaker setSendPort:appPort];
    [appSpeaker selectorRPC:"openFile:onHost:atTrueLine:"
	     paramTypes:"cci", buffer, "localhost", 0];
    [appSpeaker performRemoteMethod:"unhide"];
    [appSpeaker free];
    return self;
}

// display the contents of the main tar file

- showBaseInstall:sender
{
    char buffer[1024];
    char *appdir=(char *)[[NXBundle mainBundle] directory];
    struct stat stat_buffer;

    if (chdir(appdir)) {
	NXRunAlertPanel(NULL, "Change to application directory failed!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
    }

    if (stat("/usr/bin/gnutar", &stat_buffer)) {
	NXRunAlertPanel("Error", "/usr/bin/gnutar is missing?\nStrange environment, cannot install!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
	return self;
    }

    if (stat("cap_basedist_1.tar.gz", &stat_buffer)) {
	NXRunAlertPanel("Error", "Distribution is not complete,\n'cap_basedist_1.tar.gz' is missing!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
	return self;
    }

    if (!stat("/usr/local/lib/cap/active_zone", &stat_buffer)) {
	    [self appendString:"[CAP is already installed on your system]\n# "]; 
    }

    sprintf(buffer,"(echo \"Listing distribution...\"; cd /; /usr/bin/gnutar tzvf %s/cap_basedist_1.tar.gz)",appdir);
    [theSubprocess send:buffer withNewline:YES];

    if (chdir(TMP_DIR)) {
	NXRunAlertPanel(NULL, "Change to /tmp directory failed!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
    }

    return self;
}

// install the contents of the main tar file

- performBaseInstall:sender
{
    char buffer[1024];
    char *appdir=(char *)[[NXBundle mainBundle] directory];
    struct stat stat_buffer;

    if (chdir(appdir)) {
	NXRunAlertPanel(NULL, "Change to application directory failed!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
    }

    if (stat("/usr/bin/gnutar", &stat_buffer)) {
	NXRunAlertPanel("Error", "/usr/bin/gnutar is missing?\nStrange environment, cannot install!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
	return self;
    }

    if (stat("cap_basedist_1.tar.gz", &stat_buffer)) {
	NXRunAlertPanel("Error", "Distribution is not complete,\n'cap_basedist_1.tar.gz' is missing!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
	return self;
    }

    if (NXRunAlertPanel("Warning","I will now install the CAP distribution release 14 on your system. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
    	if (chdir(TMP_DIR)) {
		NXRunAlertPanel(NULL, "Change to /tmp directory failed!", "Ok",NULL,NULL);
		[NXApp terminate:self];
		exit(0);
    	}
	return self;
    }

    if (!stat("/usr/local/lib/cap/active_zone", &stat_buffer)) {
    	if (NXRunAlertPanel("Warning","A previous installation was detected, I'll try to save setup files, however a backup of /usr/local/cap and /usr/local/lib/cap is recommended!", "Stop","Ok",NULL)!=NX_ALERTDEFAULT) {
		if (NXRunAlertPanel("Attention","Do you want me to keep the previous setup, this will only work for previous CAPer V8 installations!", "Yes","No",NULL)==NX_ALERTDEFAULT) {
			system("/bin/mkdir /tmp/cap_save");
			system("/bin/cp /usr/local/lib/cap/active_zone /tmp/cap_save/active_zone");
			system("/bin/cp /usr/local/lib/cap/afpvols /tmp/cap_save/afpvols");
			system("/bin/cp /usr/local/lib/cap/*.conf /tmp/cap_save/");
			system("/bin/cp /usr/local/cap/start-print-servers /tmp/cap_save/");
		}
    	} else {
    		if (chdir(TMP_DIR)) {
			NXRunAlertPanel(NULL, "Change to /tmp directory failed!", "Ok",NULL,NULL);
			[NXApp terminate:self];
			exit(0);
    		}
    		return self;
	}
    }

    sprintf(buffer,"(echo \"Installing distribution...\"; cd /; /usr/bin/gnutar xzvf %s/cap_basedist_1.tar.gz; /usr/local/cap/rebuild_setup)",appdir);
    [theSubprocess send:buffer withNewline:YES];
    [theSubprocess send:"(echo \"Building Devices...\"; /usr/local/etc/builddev)" withNewline:YES];

    if (chdir(TMP_DIR)) {
	NXRunAlertPanel(NULL, "Change to /tmp directory failed!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
    }

    return self;
}

// startup CAP

- startCAP:sender
{
    if ([StatusIcon state]) {
	if (NXRunAlertPanel("Warning","CAP is running at the moment, a restart would kill all open connections! Do you really want to do this?","Cancel","Restart",NULL)==NX_ALERTDEFAULT) {
		return self;
   	}
    }
    [theSubprocess send:"(echo \"Starting CAP...\"; /usr/local/cap/kill-cap-servers; sleep 1; /usr/local/cap/start-cap-servers)" withNewline:YES];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:25000 cancelPrevious:NO];
    return self;
}

// stop & unload CAP and BPF

- unloadAll:sender
{
    if (NXRunAlertPanel("Warning","This will stop CAP completely and unload the BPF from the kernel. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
	return self;
    }
    [theSubprocess send:"(echo \"Stopping CAP...\"; /usr/local/cap/kill-cap-servers)" withNewline:YES];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:10000 cancelPrevious:NO];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:30000 cancelPrevious:NO];
    [theSubprocess send:"(echo \"Unloading the BPF LKS...\"; sleep 1; /usr/local/etc/bpf_unload)" withNewline:YES];
    return self;
}

// stop CAP but do not unload the BPF

- stopCAP:sender
{
    if (NXRunAlertPanel("Warning","Do you really want to shutdown the local CAP server?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
	return self;
    }
    [theSubprocess send:"(echo \"Stopping CAP...\"; /usr/local/cap/kill-cap-servers)" withNewline:YES];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:10000 cancelPrevious:NO];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:30000 cancelPrevious:NO];
    return self;
}

// stop CAP in 5 minutes, AUFS only

- softstopAUFS:sender
{
    if (![StatusIcon state]) {
		NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
		return self;
    }
    if (NXRunAlertPanel("Warning","Do you really want to initiate a 5 minutes shutdown for all AUFS file services?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
	return self;
    }
    [theSubprocess send:"(echo \"Stopping AUFS (in 5 minutes)...\"; /usr/local/cap/softkill-file-servers)" withNewline:YES];
    [self perform:@selector(checkForRunning:) with:NULL afterDelay:30000 cancelPrevious:NO];
    return self;
}

// show all cap processes

- listCAP:sender
{
    [self checkForRunning:self];
    [theSubprocess send:"(echo \"Showing CAP processes...\"; /usr/local/cap/list-cap-servers)" withNewline:YES];
    return self;
}

// show network entities seen by atis

- listNetwork:sender
{
    [self checkForRunning:self];
    if (![StatusIcon state]) {
	NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
	return self;
    }
    [theSubprocess send:"(echo \"Showing Network entities for local zone...\"; /usr/local/cap/atlook)" withNewline:YES];
    return self;
}

// show zone list (if router is found)

- listZones:sender
{
    [self checkForRunning:self];
    if (![StatusIcon state]) {
	NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
	return self;
    }
    [theSubprocess send:"(echo \"Querying router for zone list...\"; echo \"If you get an error here your network perhaps has no router and no zones.\"; /usr/local/cap/getzones)" withNewline:YES];
    return self;
}

// ping all entities

- pingNetwork:sender
{
    [self checkForRunning:self];
    if (![StatusIcon state]) {
	NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
	return self;
    }
    [theSubprocess send:"(echo \"Pinging Network entities...\"; /usr/local/cap/atpinger)" withNewline:YES];
    return self;
}

// list all laserwriters

- listLwriters:sender
{
    [self checkForRunning:self];
    if (![StatusIcon state]) {
	NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
	return self;
    }
    [theSubprocess send:"(echo \"Showing Network printers and queues...\"; /usr/local/cap/atlooklws)" withNewline:YES];
    return self;
}

// show the current configuration (created automatically - atalk is autoconfig)

- listEtalk:sender
{
    [self checkForRunning:self];
    [theSubprocess send:"(echo \"Showing /etc/etalk.local as written by atis...\"; /bin/cat /etc/etalk.local)" withNewline:YES];
    return self;
}

// open mount panel, delayed

- openMounter:sender
{
    [self checkForRunning:self];

    if (![StatusIcon state]) {
	NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
	return self;
    }

    [theSubprocess send:"(echo \"Searching local AppleShare fileservers...\"; /usr/local/cap/atlook | grep AFPServer | awk -F: '{print $1}'" withNewline:YES];
    [MountZone setStringValue:"*"];
    [MountPanel perform:@selector(makeKeyAndOrderFront:) with:self afterDelay:3000 cancelPrevious:YES];
    return self;
}

// mount a remote volume

- mountIt:sender
{
    char buffer[1024];
    char mapping[64];
    char RemoteSystem[1024];
    char acCmd[1024];
    FILE *pfIdent;
    struct stat stat_buffer;

    sprintf(RemoteSystem,"%s:AFPServer@%s",[MountSystem stringValue],[MountZone stringValue]);

   // attention, atident is a hack, it fails on newer authentication schemes, so provde a bailout

   if (strlen([MountUser stringValue])) {
    	sprintf(acCmd,"/usr/local/cap/atident -U%s -P%s \"%s\"",[MountUser stringValue],[MountPassword stringValue],
		RemoteSystem);
    } else {
       	sprintf(acCmd,"/usr/local/cap/atident \"%s\"",RemoteSystem); 
    }

    pfIdent=popen(acCmd,"r");

    while (!feof(pfIdent)) {
	char acLine[1024];
	acLine[0]=0;
	fgets(acLine,1023,pfIdent);
	if (!strncmp("E9",acLine,2) && (acLine[4]==':')) {
		if (NXRunAlertPanel(NULL,"There was a login problem or CAPer was unable to check your login due to an unknown authentication scheme. Do you want to try to mount the filesystem anyway?","Yes","No",NULL)!=NX_ALERTDEFAULT) {
			pclose(pfIdent);
			return self;
		}
	}
    }
    pclose(pfIdent);

    if (!stat([MountPoint stringValue], &stat_buffer)) {
    	if (NXRunAlertPanel("Warning","The MountPoint is in use, do you want to override the previous mounting?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}
    }
   if ([CharMapping state]) {
	sprintf(mapping,"-c next");
   } else {
	sprintf(mapping,"-c cap");
   }

   if ((strlen([MountSystem stringValue])+strlen([MountZone stringValue]))>1023) {
	NXBeep();
	return self;
   }

   if (strlen([MountUser stringValue])) {
   	sprintf(buffer,"(echo \"Mounting filesystem...\"; /usr/local/cap/afpmount %s -u \"%s\":\"%s\" \"%s\" %s); sleep 2", mapping,[MountUser stringValue],[MountPassword stringValue], RemoteSystem,[MountPoint stringValue]);
   } else {
   	sprintf(buffer,"(echo \"Mounting filesystem...\"; /usr/local/cap/afpmount %s \"%s\" %s); sleep 2", mapping, RemoteSystem,[MountPoint stringValue]);
   }
   [theSubprocess send:buffer withNewline:YES];
   [self showMounts:self];
   [MountPassword setStringValue:""];
   return self;
}

// show all mounts

- showMounts:sender
{
   [self perform:@selector(checkForRunning:) with:NULL afterDelay:1000 cancelPrevious:NO];
   [theSubprocess send:"echo \"Show all active afpmounts (number at end is PID):\n\"; /usr/etc/mount | grep afpmount; echo \"\nIf a filesystem comes up empty, your user/password is perhaps wrong.\"; echo \"To unmount an afpmounted filesystem issue a 'kill PID' in a shell as root\""  withNewline:YES];
   return self;	
}

// check logs, called by heartbeat

- checkLogs:Sender
{
    struct stat statbuf;
    if (!stat(FILESERVERLOG,&statbuf)) {
	static int lastFSVal;
	char acTitle[512];
	sprintf(acTitle,"File Server Log - %s - %d kBytes", FILESERVERLOG, (int)(statbuf.st_size/1024));
	[[FileServerLog window] setTitle:acTitle];

	if ((statbuf.st_size-lastFSVal)>32768) {
    		    NXStream *stream = NULL;
		    if (!(stream = NXMapFile(FILESERVERLOG, NX_READONLY))) {
			return nil;
    		    }
		    [FileServerLog readText:stream];
    		    [FileServerLog setSel:[FileServerLog textLength] :0];
		    [FileServerLog scrollSelToVisible];
		    NXClose(stream);
		    lastFSVal=statbuf.st_size;
	} else
	if (statbuf.st_size>lastFSVal) {
		FILE *pfReadMe=fopen(FILESERVERLOG,"r");
		if (pfReadMe) {
			fseek(pfReadMe, lastFSVal, SEEK_SET);
			while (!feof(pfReadMe)) {
				char acLine[1024];
				acLine[0]=0;
				fgets(acLine,1023,pfReadMe);
				if (acLine[0]!=0) {
    					[FileServerLog setSel:[FileServerLog textLength] :0];
    					[FileServerLog replaceSel:acLine];
				}
			}
			fclose(pfReadMe);
			[FileServerLog scrollSelToVisible];
		}
		lastFSVal=statbuf.st_size;
	}
    }

    if (!stat(PRINTSERVERLOG,&statbuf)) {
	static int lastPSVal;
	char acTitle[512];
	sprintf(acTitle,"Print Server Log - %s - %d kBytes", PRINTSERVERLOG, (int)(statbuf.st_size/1024));
	[[PrintServerLog window] setTitle:acTitle];

	if ((statbuf.st_size-lastPSVal)>32768) {
    		    NXStream *stream = NULL;
		    if (!(stream = NXMapFile(PRINTSERVERLOG, NX_READONLY))) {
			return nil;
    		    }
		    [PrintServerLog readText:stream];
    		    [PrintServerLog setSel:[PrintServerLog textLength] :0];
		    [PrintServerLog scrollSelToVisible];
		    NXClose(stream);
		    lastPSVal=statbuf.st_size;
	} else
	if (statbuf.st_size>lastPSVal) {
		FILE *pfReadMe=fopen(PRINTSERVERLOG,"r");
		if (pfReadMe) {
			fseek(pfReadMe, lastPSVal, SEEK_SET);
			while (!feof(pfReadMe)) {
				char acLine[1024];
				acLine[0]=0;
				fgets(acLine,1023,pfReadMe);
				if (acLine[0]!=0) {
    					[PrintServerLog setSel:[PrintServerLog textLength] :0];
    					[PrintServerLog replaceSel:acLine];
				}
			}
			fclose(pfReadMe);
			[PrintServerLog scrollSelToVisible];
		}
		lastPSVal=statbuf.st_size;
	}
    }
    [self perform:@selector(checkLogs:) with:NULL afterDelay:5000 cancelPrevious:YES];
    return self;
}

// heartbeat, restarts itself again and again

- checkForRunning:sender
{
    int nCount=0;
    FILE *pfile;
    char buffer[1024];

    sprintf(buffer,"/usr/local/cap/list-cap-servers");
    pfile=popen(buffer,"r");
    if (!pfile) {
	return self;
    }
    while (!feof(pfile)) {
	getc(pfile);
	nCount++;
    }
    pclose(pfile);
    if (nCount>10) {
	[StatusIcon setState:1];
    } else {
	[StatusIcon setState:0];
    }
    sprintf(buffer,"/usr/etc/mount | grep afpmount");
    pfile=popen(buffer,"r");
    if (!pfile) {
	return self;
    }
    while ([[MountedPopup target] count]>0) [[MountedPopup target] removeItemAt:0];
    while (!feof(pfile)) {
	char linebuffer[1024];
	linebuffer[0]=0;
	fgets(linebuffer,1023,pfile);
	if (strlen(linebuffer)) [[MountedPopup target] insertItem:linebuffer at:0];
    }
    pclose(pfile);

    [[MountedPopup target] setTarget:self];
    [[MountedPopup target] setAction:@selector(showFSData:)];

    [self perform:@selector(checkForRunning:) with:NULL afterDelay:20000 cancelPrevious:YES];

    return self;
}

// helper for umount panel

- showFSData:sender
{
    if (strncmp("afpmount",[[sender selectedCell] title],8)) return self;
    [ToBeUnmounted setStringValue:[[sender selectedCell] title]];
    return self;
}

// unmount a afpmounted fielsystem

- unmountIt:sender
{
    FILE *pfile;
    char buffer[1024];
    int nPID=(-1), nFound=0,nNum=0;
    if (strlen([ToBeUnmounted stringValue])<2) {
	NXRunAlertPanel(NULL, "Please select a filesystem from the list!", "Ok",NULL,NULL);
	return self;
    }
    sscanf([ToBeUnmounted stringValue],"afpmount-%d",&nPID);
    if (nPID<1) {
	NXRunAlertPanel(NULL, "Internal Problem parsing the mount information, giving up before the sh*t hits the fan!", "Ok",NULL,NULL);
	return self;    
    }
    sprintf(buffer,"ps -agux | grep afpmount | grep -v grep | awk '{print $2}'");
    pfile=popen(buffer,"r");
    if (!pfile) return self;
    while (!feof(pfile)) {
	char linebuffer[1024];
	linebuffer[0]=0;
	fgets(linebuffer,1023,pfile);
	if (strlen(linebuffer)>1)
        	if ((nNum=atoi(linebuffer))>1) {
			if (nNum==nPID) nFound++;
		}
    }
    pclose(pfile);
    if (nFound==1) {
   	if (NXRunAlertPanel("Warning","The selected filesystem is to be unmounted, make sure it is no longer in use before proceed!", "Make my day","Cancel",NULL)==NX_ALERTDEFAULT) {
		kill(nPID,15);
		[ToBeUnmounted setStringValue:""];
    		[self perform:@selector(checkForRunning:) with:NULL afterDelay:2000 cancelPrevious:NO];
    		[self perform:@selector(checkForRunning:) with:NULL afterDelay:10000 cancelPrevious:NO];
    	}
    } else {
	NXRunAlertPanel(NULL, "Internal Problem finding the afpmount process, please kill by hand!", "Ok",NULL,NULL);
	return self;    
    }

    return self;
}

// write the preferences

- setPrefs:sender
{
	char oldzone[1024];
	char buffer[1024];
    	FILE *pfZone;
    	char acWork[64];

    	sprintf(acWork,"%d",[AutoStartup state]);
    	NXWriteDefault("CAPer","AutoStartup", acWork);

	strcpy(oldzone,"tregeis knarf saw ereh");

    	pfZone=fopen(ZONEFILE,"r");
    	if (pfZone) {
    		fgets(oldzone,1023,pfZone);
    		fclose(pfZone);
	}

    	pfZone=fopen(ZONEFILE,"w");
    	if (!pfZone) {
		NXRunAlertPanel(NULL,"Cannot write to /usr/local/lib/cap/active_zone","Ok",NULL,NULL);
		return self;
    	}
	fputs([Zone stringValue],pfZone);
	fclose(pfZone);
    	pfZone=fopen(ZONEFILE,"r");
    	if (!pfZone) {
		NXRunAlertPanel(NULL,"Could not reread file!","Ok",NULL,NULL);
		return self;
    	}
    	fgets(buffer,1023,pfZone);
    	[Zone setStringValue:buffer];
    	fclose(pfZone);

	if (strcmp(oldzone, buffer)) {
    		if ([StatusIcon state]) {
			NXRunAlertPanel("Note","Remember you have to restart CAP to establish this machine in the new zone.","Ok",NULL,NULL);
		}
	}

	return self;
}

// open a manual page from the CAPer path in a window

- openManPage:sender
{
	char filebase[1024];
	int i;
	sprintf(filebase,"%s",[[sender selectedCell] title]);
	for (i=1;i<9;i++) {
		id ret;
		char filename[1024];
		sprintf(filename,"%d %s",i,filebase);
		ret=[ShowTextControler newFromFile:filename];
		if (ret) mainTextWindowControler=ret;
	}
	return self;
}

// helper for showText

- invalidateMainTextWindowControler:sender
{
	if (sender==mainTextWindowControler) mainTextWindowControler=NULL;
	return self;
}

- mainTextWindowControler
{
	return mainTextWindowControler;
}


// init the application, do all startup stuff

- appDidInit:sender
{
    FILE *pfZone;
    char buffer[1024];
    char *appdir=(char *)[[NXBundle mainBundle] directory];
    const char *pcWork= NXGetDefaultValue("CAPer","AutoStartup");

    struct stat stat_buffer;
    

    [[FileServerLog window] setFrameUsingName:"FileServerLog"];
    [[PrintServerLog window] setFrameUsingName:"PrintServerLog"];
    [[GeneralService window] setFrameUsingName:"Chooser"];
    [[CAPLog window] setFrameUsingName:"CAPLog"];
    [MountPanel setFrameUsingName:"MountPanel"];
    [PrinterSetupPanel setFrameUsingName:"PrinterSetupPanel"];
    [EFSWindow setFrameUsingName:"EFSWindow"];
    [EPRWindow setFrameUsingName:"EPRWindow"];
    [[ToBeUnmounted window] setFrameUsingName:"UnMountPanel"];

    if (pcWork) {
		int nValue;
		nValue=atoi(pcWork);
		[AutoStartup setState:nValue];
    }

    sprintf(activeZone,"*");

    while ([EntityList cellCount]>0) [EntityList removeRowAt:0 andFree:YES];
    [EntityList sizeToCells];
    [EntityList display];

    while ([ZoneList cellCount]>0) [ZoneList removeRowAt:0 andFree:YES];
    [ZoneList sizeToCells];
    [ZoneList display];


    if ((geteuid()==0)&&(getuid()!=0)) {
	id myNIDomain=[[NIDomain alloc] init];
	id myNILogin=[NILoginPanel new];
	[myNIDomain setConnection:"/"];
	[myNILogin setDelegate:self];
	if ([myNILogin runModalWithValidation:self inDomain:(void *)[myNIDomain getDomainHandle]
	 withUser:"root" withInstruction:"Please provide the root password to run the CAPer application." allowChange:NO]!=TRUE) {
		NXRunAlertPanel(NULL, "CAPer will now terminate.\nHave a nice day!", "Ok",NULL,NULL);
		[NXApp terminate:self];
		exit(0);
	}
    }

    if ((geteuid()==0)&&(getuid()==0)) {
    	sprintf(buffer,"/etc/chown root %s/CAPer", appdir);
    	system(buffer);
    	sprintf(buffer,"/bin/chmod 4755 %s/CAPer", appdir);
   	system(buffer);
    }

    if (geteuid()!=0) {
		NXRunAlertPanel(NULL, "Please run CAPer as root - it will then install itself SUID root (with a password check on startup).", "Ok",NULL,NULL);
		[NXApp terminate:self];
		exit(0);
    }

    setruid(0);

    if (chdir(TMP_DIR)) {
	NXRunAlertPanel(NULL, "Change to /tmp directory failed!", "Ok",NULL,NULL);
	[NXApp terminate:self];
	exit(0);
    }

    [[CAPLog window] orderFront:self];

    [self appendString:"Welcome to CAPer by Frank Siegert Release 16 V9 of 10/11/97 CAP pl198 (IP/flock patch)\nPlease read the information in the Info panel before proceeding.\n\n"];

    theSubprocess =
	[[Subprocess alloc]
	    init:"/bin/sh" withDelegate:self andPtySupport:YES andStdErr:YES];

    if (!theSubprocess) {
	NXRunAlertPanel(NULL,"Unable to start subporcess - terminating application!","Oh",NULL,NULL);
	[NXApp terminate:self];
    }

    [self checkForRunning:self];

    pfZone=fopen(ZONEFILE,"r");
    if (!pfZone) {
    	[self appendString:"Note: Some files are not in their intended place. Base installation needed!\n\n"];
	NXRunAlertPanel(NULL,"Some files are not in their intended place. Base installation needed!","Ok",NULL,NULL);
    	[Zone setStringValue:"*"];
	return self;
    }
    fgets(buffer,1023,pfZone);
    [Zone setStringValue:buffer];
    fclose(pfZone);

    if (stat("/usr/local/cap/rebuild_setup", &stat_buffer)) {
	NXRunAlertPanel(NULL, "You are running an older release of CAPer, please reinstall!", "Ok",NULL,NULL);
	[AutoStartup setState:0];
    } else
    if ((!stat("/usr/local/cap/ash", &stat_buffer))&&(stat("/usr/local/cap/atident", &stat_buffer))) {
	NXRunAlertPanel(NULL, "You are running an older release of CAPer, please reinstall! Your setup files need to be remade for this version. Do not replace them with older versions!", "Ok",NULL,NULL);
	[AutoStartup setState:0];
    } else {
	if (stat("/usr/local/lib/cap/CAPVERSION", &stat_buffer)) {
		NXRunAlertPanel(NULL, "You are running a V6 release of CAPer, please reinstall! Your setup files need to be remade for this version. Do not replace them with older versions!", "Ok",NULL,NULL);
		[AutoStartup setState:0];
	}
    }

    if ([AutoStartup state]) {
    	   [self appendString:"Note: CAP will startup in 5 seconds automatically if it is not already running.\n\n"];
	   [self perform:@selector(intelligentStartup:) with:self afterDelay:5000 cancelPrevious:YES];
    }

    [self checkLogs:self];

    return self;
}

// called before termination the applciation, NOT when logout

- appWillTerminate:sender
{
    char acWork[64];

    sprintf(acWork,"%d",[AutoStartup state]);
    NXWriteDefault("CAPer","AutoStartup", acWork);

    [[GeneralService window] saveFrameUsingName:"Chooser"];
    [[CAPLog window] saveFrameUsingName:"CAPLog"];
    [MountPanel saveFrameUsingName:"MountPanel"];
    [PrinterSetupPanel saveFrameUsingName:"PrinterSetupPanel"];
    [EFSWindow saveFrameUsingName:"EFSWindow"];
    [EPRWindow saveFrameUsingName:"EPRWindow"];
    [[ToBeUnmounted window] saveFrameUsingName:"UnMountPanel"];
    [[FileServerLog window] saveFrameUsingName:"FileServerLog"];
    [[PrintServerLog window] saveFrameUsingName:"PrintServerLog"];

    [theSubprocess terminate:sender];

    return self;
}

// helper, restart at startup

- intelligentStartup:sender
{
    if ([StatusIcon state]) return self;
    [self startCAP:self];
    return self;
}

// delegate for LoginPanel

- (BOOL)panel:thePanel authenticateUser:(const char *)userName withPassword:(const char *)password inDomain:(const void *)domain
{
	char *pass;
	char acSalt[3]={0,0,0};
	struct passwd *myPass=getpwuid(0);
	acSalt[0]=*(myPass->pw_passwd);
	acSalt[1]=*(myPass->pw_passwd+1);
	pass=crypt((char *)password, acSalt);
	return (!strcmp(pass,myPass->pw_passwd));
}

// display zones in net chooser

- updateZones:sender
{
	char acCmd[1024];

	FILE *pfCmd;
	int nCell=0;

	if (![StatusIcon state]) {
		NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
		return self;
	}

	switch ([sender tag]) {
		case 0: [PrintserverButton setState:0];
			[FileserverButton setState:1];
			[GeneralService setStringValue:"Fileserver - lookup"];
		break;
		case 1: [FileserverButton setState:0];
 			[PrintserverButton setState:1];
			[GeneralService setStringValue:"Printserver - lookup"];
		break;
	}

	currentEntityType=[sender tag];

	while ([ZoneList cellCount]>0) [ZoneList removeRowAt:0 andFree:YES];
	[ZoneList sizeToCells];
	[ZoneList display];
	DPSFlush();

	sprintf(acCmd,"/usr/local/cap/getzones 2> /dev/null | sort");
	pfCmd=popen(acCmd,"r");
	while (!feof(pfCmd)) {
		id theCellFromMatrix;
		char acBack[1024];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if (strlen(acBack)>1) {
			int i;
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;

			[ZoneList addRow];
			theCellFromMatrix=[ZoneList cellAt:nCell++ :0];
			[theCellFromMatrix setTitle:acBack];
			[theCellFromMatrix setTarget:self];
			[theCellFromMatrix setAction:@selector(updateEntities:)];
		}
	}
	[ZoneList sizeToCells];
	[ZoneList display];
	pclose(pfCmd);

	switch ([sender tag]) {
		case 0: 
			[GeneralService setStringValue:"Fileserver"];
		break;
		case 1: 
			[GeneralService setStringValue:"Printserver"];
		break;
	}

	if (nCell==0) {
		[self updateEntities:self];
	}

	return self;
}

// display entities in current zone in net chooser

- updateEntities:sender
{
	char acCmd[1024];


	FILE *pfCmd;
	int nCell=0;

	char currentZone[256];
	currentZone[0]=0;

	if ([ZoneList cellCount]>0) {
		sprintf(currentZone,"%s",[[ZoneList selectedCell] title]);
	}

	while ([EntityList cellCount]>0) [EntityList removeRowAt:0 andFree:YES];
	[EntityList sizeToCells];
	[EntityList display];
	switch (currentEntityType) {
		case 0: /* fileserver */
		sprintf(acCmd,"/usr/local/cap/atlook \"%s\" | grep AFPServer | awk -F: '{print $1}' | sort",currentZone);
		break;
		case 1: /* printer */
		sprintf(acCmd,"/usr/local/cap/atlook \"%s\" | grep LaserWriter | awk -F: '{print $1}' |sort", currentZone);
		break;
		default: exit(-2);
	}
	pfCmd=popen(acCmd,"r");
	while (!feof(pfCmd)) {
		id theCellFromMatrix;
		char acBack[1024];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if (strlen(acBack)>1) {
			int i;
			char *pcEntity=index(acBack,'-');
			if (!pcEntity) pcEntity=acBack; else {
				if (*(pcEntity+1)) pcEntity++;
				if (*(pcEntity+1)) pcEntity++;
			}
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;
			[EntityList addRow];
			theCellFromMatrix=[EntityList cellAt:nCell++ :0];
			if (pcEntity[0]==32) 
				[theCellFromMatrix setTitle:pcEntity+1];
			else
				[theCellFromMatrix setTitle:pcEntity];
			[theCellFromMatrix setTarget:self];
			[theCellFromMatrix setAction:@selector(editEntities:)];

		}
	}
	[EntityList sizeToCells];
	[EntityList display];
	pclose(pfCmd);
	return self;
}

// user reqeusted a different type of entitiy display

- editEntities:sender
{
	char afpname[1024];
	char currentZone[256];
	currentZone[0]=0;

	if ([ZoneList cellCount]>0) {
		sprintf(currentZone,"%s",[[ZoneList selectedCell] title]);
	} else sprintf(currentZone,"*");

	strcpy(activeZone, currentZone);

	switch(currentEntityType) {
	case 0: /* fileserver */
		[MountName setStringValue:[[sender selectedCell] title]];
		[MountZone setStringValue:currentZone];
   		[MountPanel perform:@selector(makeKeyAndOrderFront:) with:self afterDelay:50 cancelPrevious:YES];
		break;
	case 1: /* printer */
		sprintf(afpname,"%s:LaserWriter@%s",[[sender selectedCell] title], currentZone);
		[PrinterAFPName setStringValue:afpname];
		[PrinterSetupPanel perform:@selector(makeKeyAndOrderFront:) with:self afterDelay:50 cancelPrevious:YES];
		[self setupLocalPrinters:self];
		[self perform:@selector(updatePrinter:) with:PrinterList afterDelay:200 cancelPrevious:YES];

	}
	return self;
}

// helper for local printer installation

- setupLocalPrinters:sender
{


	char acCmd[1024];
	FILE *pfCmd;
	int nCell=0;

	while ([PrinterList cellCount]>0) [PrinterList removeRowAt:0 andFree:YES];
	[PrinterList sizeToCells];
	[PrinterList display];
	DPSFlush();

	sprintf(acCmd,"niutil -list . /printers | awk '{print $2}'");
	pfCmd=popen(acCmd,"r");
	while (!feof(pfCmd)) {
		id theCellFromMatrix;
		char acBack[1024];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if (strlen(acBack)>1) {
			int i;
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;

			[PrinterList addRow];
			theCellFromMatrix=[PrinterList cellAt:nCell++ :0];
			[theCellFromMatrix setTitle:acBack];
			[theCellFromMatrix setTarget:self];
			[theCellFromMatrix setAction:@selector(updatePrinter:)];
		}
	}
	[PrinterList sizeToCells];
	[PrinterList display];
	pclose(pfCmd);

	return self;
}

// update the printer display

- updatePrinter:sender
{
	char acCmd[1024];
	FILE *pfCmd;
	sprintf(acCmd,"niutil -read . /printers/%s",[[sender selectedCell] title]);
	pfCmd=popen(acCmd,"r");
	sprintf(activePrinter,"%s",[[sender selectedCell] title]);

	[PrinterRevertButton setEnabled:NO];
	[PrinterConnectButton setEnabled:NO];
	[PrinterTest setEnabled:NO];
	while (!feof(pfCmd)) {
		char acBack[1024];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if (strlen(acBack)>1) {
			int i;
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;

			if (!strncmp("if: ",acBack,4)) {
				if (!strncmp("/usr/lib/NextPrinter/Server/prserver",acBack+4,36)) {
					[PrinterNote setStringValue:"Printer is served by prserver\n(standard methode for normal printers)"];
					if (strlen([PrinterAFPName stringValue])>0)
						[PrinterConnectButton setEnabled:YES];
					[PrinterTest setEnabled:YES];
				} else
				if (!strncmp("/usr/local/cap/",acBack+4,15)) {
					char acLine[1024], acBuild[1024];
					FILE *pfRead=fopen(acBack+4,"r");
					sprintf(acLine,"Strange - no control file found");
					if (pfRead) {
						fgets(acLine,1023,pfRead);
						fgets(acLine,1023,pfRead);
						acLine[0]=0;
						fgets(acLine,1023,pfRead);
						for (i=strlen(acLine);i>0;i--) if (acLine[i]<32) acLine[i]=0;

						fclose(pfRead);
					}
					if (!strncmp("# For: ", acLine,7))
						sprintf(acBuild,"Printer is served by CAP\n(%s)",acLine+7);
					else
						sprintf(acBuild,"Printer is served by CAP\n(%s)",acLine);

					[PrinterNote setStringValue:acBuild];
					[PrinterRevertButton setEnabled:YES];
					[PrinterTest setEnabled:YES];
				} else {
					[PrinterNote setStringValue:acBack+4];
					[PrinterTest setEnabled:YES];
				}
			}
		}
	}
	pclose(pfCmd);
	return self;
}

// turn a printer into a appletalked queue (NeXT -> LW)

- makeCAPPrinter:sender
{
	FILE *pfMakePrinter;
	char acMakePrinter[1024];
	char acNisetup[1024];

	if (strlen(activePrinter)<1) return self;

	sprintf(acMakePrinter,"/usr/local/cap/%s.sh", activePrinter);

	pfMakePrinter=fopen(acMakePrinter,"w");

	if (!pfMakePrinter) {
		NXRunAlertPanel(NULL,"Unable to write to /usr/local/cap!","Ok",NULL,NULL);
	}

	fprintf(pfMakePrinter,"#!/bin/sh\n");
	fprintf(pfMakePrinter,"# Automatically setup by CAPer V5\n");
	fprintf(pfMakePrinter,"# For: %s\n",[PrinterAFPName stringValue]);
	fprintf(pfMakePrinter,"/usr/local/cap/lwpr -p \"%s\" stdin\n",[PrinterAFPName stringValue]);
	fclose(pfMakePrinter);

	sprintf(acNisetup,"chmod a+x %s", acMakePrinter);
	system(acNisetup);

	sprintf(acNisetup,"niutil -createprop . /printers/%s if %s", activePrinter, acMakePrinter);
	system(acNisetup);

	[self perform:@selector(updatePrinter:) with:PrinterList afterDelay:200 cancelPrevious:YES];

	return self;
}

// revert printer to normal

- makeNormalPrinter:sender
{
	char acNisetup[1024];

	if (strlen(activePrinter)<1) return self;

	sprintf(acNisetup,"niutil -createprop . /printers/%s if /usr/lib/NextPrinter/Server/prserver", activePrinter);
	system(acNisetup);

	[self perform:@selector(updatePrinter:) with:PrinterList afterDelay:200 cancelPrevious:YES];

	return self;
}

// test the queue

- testPrinter:sender
{
	char acPrintTest[1024];
	sprintf(acPrintTest,"/usr/ucb/lpr -P%s /usr/lib/NextPrinter/English.lproj/testpage.ps", activePrinter);

    	if (NXRunAlertPanel("Confirm","This will print a testpage on the selected printer using the UNIX queue. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}

	system(acPrintTest);
	return self;
}

// test PAP connection

- testAFPprinter:sender
{
	char acPrintTest[1024];
	sprintf(acPrintTest,"/usr/local/cap/lwpr -p \"%s\" /usr/lib/NextPrinter/English.lproj/testpage.ps", [PrinterAFPName stringValue]);

	if (![StatusIcon state]) {
		NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
		return self;
	}

    	if (NXRunAlertPanel("Confirm","This will print a testpage on the selected printer using only the Appletalk layer. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}
	printf("%s\n", acPrintTest);
	system(acPrintTest);
	return self;
}

// restart AUFS

- restartAUFS:sender
{
	if (![StatusIcon state]) {
		NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
		return self;
	}

	if (NXRunAlertPanel("Confirm","The CAP fileserver is to be restarted. All connections are terminated! Do you really want to do this?", "Skip","Make my day",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}
	[theSubprocess send:"/usr/local/cap/restart-file-servers"  withNewline:YES];
	return self;
}

// restart printing

- restartPrinting:sender
{
	if (![StatusIcon state]) {
		NXRunAlertPanel("Warning","CAP is not running, unable to access Appletalk!","Ok",NULL,NULL);
		return self;
	}

	if (NXRunAlertPanel("Confirm","The CAP printserver is to be restarted. All incoming connections are terminated! Do you really want to do this?", "Skip","Make my day",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}
	[theSubprocess send:"/usr/local/cap/restart-print-servers"  withNewline:YES];
	return self;
}

// EPR are the imported printers (Mac -> NEXT)

- openDefinePrinters:sender
{
	[self readEPR:self];
	[self updateEPRList:self];
	[EPRWindow makeKeyAndOrderFront:self];
	return self;
}

- updateEPRList:sender
{
	int nCell=0,i;

	while ([EPRList cellCount]>0) [EPRList removeRowAt:0 andFree:YES];
	[EPRList sizeToCells];
	[EPRList display];
	DPSFlush();

	for (i=0;i<[EPRInternalList count];i++) {
		char *mydata=[(CAPStorage *)[[EPRInternalList objectAt:i] objectAt:0] data];
		id theCellFromMatrix;
		[EPRList addRow];
		theCellFromMatrix=[EPRList cellAt:nCell++ :0];
		if (mydata) [theCellFromMatrix setTitle:mydata];
		[theCellFromMatrix setTarget:self];
		[theCellFromMatrix setAction:@selector(updateEPR:)];

	}
	[EPRList sizeToCells];
	[EPRList display];

	[self perform:@selector(updateEPR:) with:EFSList afterDelay:200 cancelPrevious:YES];

	return self;
}

- updateEPR:sender
{
	int row;
	for (row=0;row<[EPRList cellCount];row++) {
		if ([EPRList cellAt:row:0]==[EPRList selectedCell]) {
			[EPRName setStringValue:[(CAPStorage *)[[EPRInternalList objectAt:row] objectAt:0] data]];
			[EPRInclude setStringValue:[(CAPStorage *)[[EPRInternalList objectAt:row] objectAt:1] data]];
			[EPRQueue setStringValue:[(CAPStorage *)[[EPRInternalList objectAt:row] objectAt:2] data]];
			return self;
		}
	}
	return self;
}

- saveEPR:sender
{
	FILE *pfWriteCmd;
	int i;

   	if (NXRunAlertPanel("Confirm","This will save the list of exported printers. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}


	pfWriteCmd=fopen("/usr/local/cap/start-print-servers","w");
	if (!pfWriteCmd) {
		NXRunAlertPanel("Warning","Cannot write to /usr/local/cap/start-print-servers","Ok",NULL,NULL);
		return self;
	}
	fprintf(pfWriteCmd,"#!/bin/sh\n# Automatically generated by CAPer\n\n");
	for (i=0;i<[EPRInternalList count];i++) {
		FILE *pfNew;
		char acNewName[1024];

		fprintf(pfWriteCmd,"/usr/local/cap/lwsrv8 /usr/local/lib/cap/lwsrv_%d.conf\n",i);
		sprintf(acNewName,"/usr/local/lib/cap/lwsrv_%d.conf",i);

		pfNew=fopen(acNewName,"w");
		if (!pfNew) {
			NXRunAlertPanel("Warning","Cannot write to %s","Ok",NULL,NULL, acNewName);
			return self;
		}

		fprintf(pfNew,"# Automatically generated by CAPer\n\nLibrary = /usr/local/lib/cap/DB;\n\n");
		fprintf(pfNew,"Options = (\n");
		fprintf(pfNew,"\tDontCollect;\n");
		fprintf(pfNew,"\tProcsetDir /usr/local/lib/cap/procsets;\n");
		fprintf(pfNew,"\tLogFile %s;\n",PRINTSERVERLOG);
		fprintf(pfNew,"\tFontFile /usr/local/lib/cap/LW+Fonts;\n");
		fprintf(pfNew,");\n\n");

		fprintf(pfNew,"\"%s\" = (\n",[(CAPStorage *)[[EPRInternalList objectAt:i] objectAt:0] data]);
		fprintf(pfNew,"\tinclude \"%s\";\n",[(CAPStorage *)[[EPRInternalList objectAt:i] objectAt:1] data]);
		fprintf(pfNew,"\tprinterqueue %s;\n",[(CAPStorage *)[[EPRInternalList objectAt:i] objectAt:2] data]);
		fprintf(pfNew,");\n\n");
		fclose(pfNew);
	}
	fclose(pfWriteCmd);
	system("chmod a+x /usr/local/cap/start-print-servers");

	if ([StatusIcon state])
		[self perform:@selector(restartPrinting:) with:self afterDelay:1000 cancelPrevious:YES];

	return self;
}

- newEPR:sender
{
	id newList=[[List alloc] init];
	CAPStorage *NameElement;

	if (!EPRInternalList) EPRInternalList=[[List alloc] init];

	if (!strlen([EPRName stringValue])) {
		NXRunAlertPanel("Warning","No Appletalk name - You must provide a name","Ok",NULL,NULL);
		return self;
	}

	if (!strlen([EPRInclude stringValue])) {
		NXRunAlertPanel("Warning","No printer type - You must provide a type","Ok",NULL,NULL);
		return self;
	}

	if (!strlen([EPRQueue stringValue])) {
		NXRunAlertPanel("Warning","No queue name - You must provide a queue name","Ok",NULL,NULL);
		return self;
	}

	NameElement=[[CAPStorage alloc] init];
	[NameElement setName:"AFPName"];
	[NameElement setData:(char *)[EPRName stringValue]];
	[newList addObject:NameElement];

	NameElement=[[CAPStorage alloc] init];
	[NameElement setName:"include"];
	[NameElement setData:(char *)[EPRInclude stringValue]];
	[newList addObject:NameElement];

	NameElement=[[CAPStorage alloc] init];
	[NameElement setName:"printerqueue"];
	[NameElement setData:(char *)[EPRQueue stringValue]];
	[newList addObject:NameElement];

	[EPRInternalList addObject:newList];
	[self updateEPRList:self];

	return self;
}

- deleteEPR:sender
{
	int row;
	for (row=0;row<[EPRList cellCount];row++) {
		if ([EPRList cellAt:row:0]==[EPRList selectedCell]) {
			[[[EPRInternalList removeObjectAt:row] freeObjects] free];
			[self updateEPRList:self];
			return self;
		}
	}
	return self;
}

- editEPR:sender
{
	int row;
	for (row=0;row<[EPRList cellCount];row++) {
		if ([EPRList cellAt:row:0]==[EPRList selectedCell]) {
			id EPRElement=[EPRInternalList objectAt:row];
			[[EPRElement objectAt:0] setData:(char *)[EPRName stringValue]];
			[[EPRElement objectAt:1] setData:(char *)[EPRInclude stringValue]];
			[[EPRElement objectAt:2] setData:(char *)[EPRQueue stringValue]];
			[self updateEPRList:self];
			return self;
		}
	}
	return self;
}

- readEPR:sender
{
	FILE *pfCmd;

	if (!EPRInternalList) EPRInternalList=[[List alloc] init];

	while ([EPRInternalList count]) {
		[[[EPRInternalList removeObjectAt:0] freeObjects] free];
	}

	pfCmd=fopen("/usr/local/cap/start-print-servers","r");
	if (!pfCmd) {
		NXBeep();
		return self;
	}
	while (!feof(pfCmd)) {
		char acBack[4096];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if ((strlen(acBack)>1)&&(acBack[0]!='#')) {
			int i;
			char *firstChar;
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;

			firstChar=index(acBack+1,' ');
			if (firstChar) {
				FILE *pfReadConfig=fopen(firstChar+1,"r");
				if (pfReadConfig) {
					int nParse=0;
					id newList=[[List alloc] init];
					while (!feof(pfReadConfig)) {
						char acLine[1024];
						acLine[0]=0;
						fgets(acLine,1023,pfReadConfig);
						for (i=strlen(acLine);i>0;i--) if (acLine[i]<32) acLine[i]=0;
						if ((acLine[0]=='"')&&(!nParse)) {
							char *nextChar=index(acLine+1,'"');
							if (nextChar) {
								CAPStorage *NameElement=[[CAPStorage alloc] init];
								[NameElement setName:"AFPName"];
								*nextChar=0;
								[NameElement setData:acLine+1];
								[newList addObject:NameElement];
								nParse++;
							}
						}
						if (nParse) {
							int j;
							for(j=0;j<strlen(acLine);j++) if (acLine[j]>32) break;
							if (!strncmp(&acLine[j],"include",7)) {
								char *theEnd;
								CAPStorage *NameElement=[[CAPStorage alloc] init];
								[NameElement setName:"include"];
								theEnd=index((&acLine[j])+9,'"');
								if (theEnd) *theEnd=0;
								[NameElement setData:(&acLine[j])+9];
								[newList addObject:NameElement];
							}
							if (!strncmp(&acLine[j],"printerqueue",12)) {
								char *theEnd;
								CAPStorage *NameElement=[[CAPStorage alloc] init];
								[NameElement setName:"printerqueue"];
								theEnd=index((&acLine[j])+13,';');
								if (theEnd) *theEnd=0;
								[NameElement setData:(&acLine[j])+13];
								[newList addObject:NameElement];
							}
						}
					}
					fclose(pfReadConfig);
					[EPRInternalList addObject:newList];
				}
			}
		}
	}
	fclose(pfCmd);


    	pfCmd=fopen("/usr/local/lib/cap/DBfile","r");
    	if (!pfCmd) return self;
    	while ([[EPRTypes target] count]>0) [[EPRTypes target] removeItemAt:0];
    	while (!feof(pfCmd)) {
		char linebuffer[1024];
		linebuffer[0]=0;
		fgets(linebuffer,1023, pfCmd);
		if (strlen(linebuffer)) {
			if (!recmp(" = (",linebuffer)) {
				char *pcLine=index(linebuffer+1,'=');
				if (pcLine) {
					char acPrinter[1024];
					*pcLine=0;
					if (linebuffer[0]=='"') {
						strcpy(acPrinter,linebuffer+1);
						pcLine=rindex(acPrinter,'"');
						if (pcLine) *pcLine=0;
					} else strcpy(acPrinter,linebuffer);
					if ((strlen(acPrinter)>0)&&(acPrinter[strlen(acPrinter)-1]==32)) 
						acPrinter[strlen(acPrinter)-1]=0;
					[[EPRTypes target] insertItem:acPrinter at:0];
					//printf("'%s'\n", acPrinter);
				}
			}
		}
    	}
    	fclose(pfCmd);

   	[[EPRTypes target] setTarget:self];
  	[[EPRTypes target] setAction:@selector(showEPRType:)];

    	pfCmd=popen("niutil -list . /printers | awk '{print $2}'","r");
    	if (!pfCmd) return self;
    	while ([[EPRUnixQueues target] count]>0) [[EPRUnixQueues target] removeItemAt:0];
    	while (!feof(pfCmd)) {
		int i;
		char linebuffer[1024];
		linebuffer[0]=0;
		fgets(linebuffer,1023, pfCmd);
		for (i=strlen(linebuffer);i>0;i--) if (linebuffer[i]<32) linebuffer[i]=0;
		if (strlen(linebuffer)) {
			[[EPRUnixQueues target] insertItem:linebuffer at:0];
		}
    	}
    	pclose(pfCmd);

   	[[EPRUnixQueues target] setTarget:self];
  	[[EPRUnixQueues target] setAction:@selector(showEPRUnixQueues:)];


	return self;
}

- showEPRType:sender
{
    if (!strcmp([[sender selectedCell] title],"Select a printer type here")) return self;
    return ([EPRInclude setStringValue:[[sender selectedCell] title]]);
}

- showEPRUnixQueues:sender
{
    if (!strcmp([[sender selectedCell] title],"Select a UNIX queue here")) return self;
    return ([EPRQueue setStringValue:[[sender selectedCell] title]]);
}

- openDefineExports:sender
{
	FILE *pfCmd;
	int nCell=0;

	while ([EFSList cellCount]>0) [EFSList removeRowAt:0 andFree:YES];
	[EFSList sizeToCells];
	[EFSList display];
	DPSFlush();

	[EFSWindow makeKeyAndOrderFront:self];

	pfCmd=fopen("/usr/local/lib/cap/afpvols","r");
	while (!feof(pfCmd)) {
		id theCellFromMatrix;
		char acBack[1024];
		acBack[0]=0;
		fgets(acBack,1023,pfCmd);
		if ((strlen(acBack)>1)&&(acBack[0]!='#')) {
			int i;
			for (i=strlen(acBack);i>0;i--) if (acBack[i]<32) acBack[i]=0;

			[EFSList addRow];
			theCellFromMatrix=[EFSList cellAt:nCell++ :0];
			[theCellFromMatrix setTitle:acBack];
			[theCellFromMatrix setTarget:self];
			[theCellFromMatrix setAction:@selector(updateEFS:)];
		}
	}
	[EFSList sizeToCells];
	[EFSList display];
	fclose(pfCmd);

	[self perform:@selector(updateEFS:) with:EFSList afterDelay:200 cancelPrevious:YES];

	return self;

}

- updateEFS:sender
{
	char *pcElement,*pcElement2,acElement[1024];
	sprintf(acElement,"%s",[[sender selectedCell] title]);
	pcElement=index(acElement,':');
	if (!pcElement) {
		return self;
	}
	pcElement2=rindex(pcElement+1,':');
	if (!pcElement2) {
		return self;
	}
	*pcElement=0;
	*pcElement2=0;
	[EFSPath setStringValue:acElement];
	[EFSName setStringValue:pcElement+1];
	return self;
}

- newEFSElement:sender
{
   	struct stat stat_buf;
	id theCellFromMatrix;
	char acElement[1024];
	char acElement2[1024];

	if (strlen([EFSPath stringValue])<1) {
		NXRunAlertPanel("Warning","Path is empty - You must provide a path","Ok",NULL,NULL);
		return self;
	}

	if (strlen([EFSName stringValue])<1) {
		NXRunAlertPanel("Warning","Name is empty - You must provide an AFP Name","Ok",NULL,NULL);
		return self;
	}

	if ((strlen([EFSName stringValue])+strlen([EFSName stringValue]))>1023) {
		NXRunAlertPanel("Warning","Path and Name together too long!","Ok",NULL,NULL);
		return self;
	}

	if (stat([EFSPath stringValue],&stat_buf)) {
		NXRunAlertPanel("Warning","Path not accessable, please check:\n%s","Ok",NULL,NULL,[EFSPath stringValue]);
		return self;
	}
	if ((stat_buf.st_mode&S_IFDIR)==0) {
		NXRunAlertPanel("Warning","Path not a directory, please check:\n%s","Ok",NULL,NULL,[EFSPath stringValue]);
		return self;
	}

	sprintf(acElement,"%s/.resource",[EFSPath stringValue]);
	sprintf(acElement2,"%s/.finderinfo",[EFSPath stringValue]);

	if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
		if (NXRunAlertPanel("Confirm","No special CAP .resource and/or .finderinfo directory found, should I create it?","Yes","No",NULL)==NX_ALERTDEFAULT) {
			char acCmd[1024];
			mkdir(acElement,0);
			mkdir(acElement2,0);
			// to rely on system is suboptimal but works for now...
			sprintf(acCmd,"chmod 777 %s", acElement);
			system(acCmd);
			sprintf(acCmd,"chmod 777 %s", acElement2);
			system(acCmd);
			if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
				NXRunAlertPanel("Warnung","Unable to create these directories!","Ok",NULL,NULL);
				return self;
			}
		}
	}

	sprintf(acElement,"%s/../.resource",[EFSPath stringValue]);
	sprintf(acElement2,"%s/../.finderinfo",[EFSPath stringValue]);

	if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
		if (NXRunAlertPanel("Confirm","No special CAP .resource and/or .finderinfo TOPLEVEL directory found, should I create it?","Yes","No",NULL)==NX_ALERTDEFAULT) {
			char acCmd[1024];
			mkdir(acElement,0);
			mkdir(acElement2,0);
			// to rely on system is suboptimal but works for now...
			sprintf(acCmd,"chmod 777 %s", acElement);
			system(acCmd);
			sprintf(acCmd,"chmod 777 %s", acElement2);
			system(acCmd);
			if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
				NXRunAlertPanel("Warnung","Unable to create these directories!","Ok",NULL,NULL);
				return self;
			}
		}
	}

	sprintf(acElement,"%s:%s:",[EFSPath stringValue],[EFSName stringValue]);
	[EFSList addRow];
	theCellFromMatrix=[EFSList cellAt:([EFSList cellCount]-1) :0];
	[theCellFromMatrix setTitle:acElement];
	[theCellFromMatrix setTarget:self];
	[theCellFromMatrix setAction:@selector(updateEFS:)];
	[EFSList sizeToCells];
	[EFSList display];

	[self perform:@selector(updateEFS:) with:EFSList afterDelay:200 cancelPrevious:YES];

	return self;
}

- deleteEFSElement:sender
{
	int row;
	for (row=0;row<[EFSList cellCount];row++) {
		if ([EFSList cellAt:row:0]==[EFSList selectedCell]) {
			[EFSList removeRowAt:row andFree:YES];
			[EFSList sizeToCells];
			[EFSList display];
			[self perform:@selector(updateEFS:) with:EFSList afterDelay:200 cancelPrevious:YES];

			return self;
		}
	}
	return self;
}

- editEFSElement:sender
{
	int row;
   	struct stat stat_buf;
	char acElement[1024];
	char acElement2[1024];

	if (strlen([EFSPath stringValue])<1) {
		NXRunAlertPanel("Warning","Path is empty - You must provide a path","Ok",NULL,NULL);
		return self;
	}

	if (strlen([EFSName stringValue])<1) {
		NXRunAlertPanel("Warning","Name is empty - You must provide an AFP Name","Ok",NULL,NULL);
		return self;
	}

	if ((strlen([EFSName stringValue])+strlen([EFSName stringValue]))>1023) {
		NXRunAlertPanel("Warning","Path and Name together too long!","Ok",NULL,NULL);
		return self;
	}

	if (stat([EFSPath stringValue],&stat_buf)) {
		NXRunAlertPanel("Warning","Path not accessable, please check:\n%s","Ok",NULL,NULL,[EFSPath stringValue]);
		return self;
	}
	if ((stat_buf.st_mode&S_IFDIR)==0) {
		NXRunAlertPanel("Warning","Path not a directory, please check:\n%s","Ok",NULL,NULL,[EFSPath stringValue]);
		return self;
	}

	sprintf(acElement,"%s/.resource",[EFSPath stringValue]);
	sprintf(acElement2,"%s/.finderinfo",[EFSPath stringValue]);

	if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
		if (NXRunAlertPanel("Confirm","No special CAP .resource and/or .finderinfo directory found, should I create it?","Yes","No",NULL)==NX_ALERTDEFAULT) {
			char acCmd[1024];
			mkdir(acElement,0);
			mkdir(acElement2,0);
			sprintf(acCmd,"chmod 777 %s", acElement);
			system(acCmd);
			sprintf(acCmd,"chmod 777 %s", acElement2);
			system(acCmd);
			if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
				NXRunAlertPanel("Warnung","Unable to create these directories!","Ok",NULL,NULL);
				return self;
			}
		}
	}

	sprintf(acElement,"%s/../.resource",[EFSPath stringValue]);
	sprintf(acElement2,"%s/../.finderinfo",[EFSPath stringValue]);

	if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
		if (NXRunAlertPanel("Confirm","No special CAP .resource and/or .finderinfo TOPLEVEL directory found, should I create it?","Yes","No",NULL)==NX_ALERTDEFAULT) {
			char acCmd[1024];
			mkdir(acElement,0);
			mkdir(acElement2,0);
			sprintf(acCmd,"chmod 777 %s", acElement);
			system(acCmd);
			sprintf(acCmd,"chmod 777 %s", acElement2);
			system(acCmd);
			if (stat(acElement,&stat_buf)||stat(acElement,&stat_buf)) {
				NXRunAlertPanel("Warnung","Unable to create these directories!","Ok",NULL,NULL);
				return self;
			}
		}
	}

	sprintf(acElement,"%s:%s:",[EFSPath stringValue],[EFSName stringValue]);

	for (row=0;row<[EFSList cellCount];row++) {
		if ([EFSList cellAt:row:0]==[EFSList selectedCell]) {
			[[EFSList selectedCell] setTitle:acElement];
			[self perform:@selector(updateEFS:) with:EFSList afterDelay:20 cancelPrevious:YES];
			return self;
		}
	}

	return self;
}

- saveEFSList:sender
{
	int row;
	FILE *pfSave;

   	if (NXRunAlertPanel("Confirm","This will save the list of exported filesystems. Do you really want to do this?", "No","Yes",NULL)==NX_ALERTDEFAULT) {
		return self;
    	}

	pfSave=fopen("/usr/local/lib/cap/afpvols","w");
	if (!pfSave) {
		NXRunAlertPanel("Warning","Unable to write to /usr/local/lib/cap/afpvols","Ok",NULL,NULL);
		return self;
	}
	fprintf(pfSave,"# Automatically generated by CAPer at [%d]\n",(unsigned int)time(0));
	for (row=0;row<[EFSList cellCount];row++) {
		id Element=[EFSList cellAt:row:0];
		fprintf(pfSave,"%s\n",[Element title]);
	}
	fclose(pfSave);
	[self perform:@selector(openDefineExports:) with:self afterDelay:200 cancelPrevious:YES];
	if ([StatusIcon state])
		[self perform:@selector(restartAUFS:) with:self afterDelay:1000 cancelPrevious:YES];
	return self;
}

- chooseEFSDir:sender
{
    	id OpenDialog=[OpenPanel new]; /* shared instance */

    	/* set up the OpenPanel and run it for a single file */
    	[OpenDialog allowMultipleFiles:NO];
	[OpenDialog chooseDirectories:YES];
    	[OpenDialog setTitle:"Directory to be exported"];   
    	if (![OpenDialog runModalForDirectory:"" file:""]) {
    		return self;
    	}
	[EFSPath setStringValue:[OpenDialog filename]];
	return self;
}

- giveLogin:sender
{
	return ([NXApp stopModal]);
}

// suboptimal mailer, use Misckit next time

- mailAway:sender
{
    	int fd;     // File descriptor
    	NXStream *stream = NULL;
    	BOOL saveOK;
	char filename[1024];
	char cmd[1024];
	if ((!strlen([MailName stringValue]) || (strlen([MailName stringValue])>512))) {
		NXRunAlertPanel(NULL,"Need a valid mail address!","Ok",NULL,NULL);
		return self;
	}
	sprintf(filename,"/tmp/sendcapmail.txt");
	unlink(filename);
	 if (saveOK = 
	    (((fd = open (filename, O_WRONLY|O_CREAT|O_TRUNC, 0644)) != -1) &&
	    (stream = NXOpenFile (fd, NX_WRITEONLY))))
	[MailText writeText:stream];
    	if (stream) NXClose (stream);
   	if (fd != -1) close (fd);

	sprintf(cmd,"/usr/ucb/mail -s \"Attn: CAPer R12V5 Feedback\" %s < %s",
		[MailName stringValue],filename);

	system(cmd);
	[[MailName window] orderOut:self];

	return self;
	
}

/*==========================================================
 *
 * New method for subclass
 *
 *==========================================================*/
 
- appendString:(char *)buffer
    // append the buffer to the end of the text object
{
    [CAPLog setSel:[CAPLog textLength] :0];
    [CAPLog replaceSel:buffer];
    [CAPLog scrollSelToVisible];
    return self;
}

/*==========================================================
 *
 * Subprocess Delegation
 *
 *==========================================================*/
 
- subprocessOutput:(char *)buffer
{
    [self appendString:buffer];
    return self;
}

- subprocessDone
{
   return self;
}

- subprocessError:(const char *)errorString
{
    NXRunAlertPanel("Error", errorString, "Ok",NULL,NULL);
    return self;
}


@end
